/*
 * File:  Motor Test
 * Name:  Alex Pippin
 * 
 * Description:
 *  This program uses 3 ADC inputs to control the position of the platform held by 4 strings in the corners 
 *  connected to 1 DC toy motor per string. The ADC inputs are used to control the x, y, and z destination
 *  coordinates in comparison to the current calculated position of the platform.
 *
 * Inputs:
 *  PORTF.0   ->  ADC   -> x position
 *  PORTF.1   ->  ADC   -> y position
 *  PORTE.4   ->  INT4  -> Pushbutton -> Update position
 * 
 * Outputs:
 *  OC1A  ->  PORTB.5  PIN11  ->  Back Left   ->  FORWARD
 *  OC1B  ->  PORTB.6  PIN12  ->  Back Left   ->  REVERSE                      Y
 *  OC3A  ->  PORTE.3  PIN 5  ->  Back Right  ->  FORWARD       |BL         BR|
 *  OC3B  ->  PORTE.4  PIN 2  ->  Back Right  ->  REVERSE     Z |     Top     |
 *  OC4A  ->  PORTH.3  PIN 6  ->  Front Left  ->  FORWARD       |     View    |
 *  OC4B  ->  PORTH.4  PIN 7  ->  Front Left  ->  REVERSE       |FL         FR|
 *  OC3C  ->  PORTE.5  PIN 3  ->  Front Right ->  FORWARD               X
 *  OC1C  ->  PORTH.5  PIN13  ->  Front Right ->  REVERSE
 *  PORTA.4:7 ->  
 */

#include <avr/io.h>
#include <avr/interrupt.h>

// Function Prototypes
void init_PORTS();
void init_ADC();
uint16_t read_ADC(uint8_t);
void init_PWM1();
void init_PWM3();
void init_PWM4();

// Global Variables
volatile uint8_t dir = 0;

int main(void) {
  // Initializations
  init_PORTS();
  init_ADC();
  init_PWM1();
  init_PWM3();

  uint16_t pwm1a_val = 0;
  uint16_t pwm1b_val = 0;
  
  while(1) {
    pwm1a_val = 4 * read_ADC(0) + 800;  // Height Servo
    pwm1b_val = 4 * read_ADC(1) + 800;  // Plunger Servo
    
    if (dir) {  // Forward
      dir = 0;
      
      // Back Left
      OCR1A = pwm1a_val;
      OCR1B = 0;
      
      // Back Right
      OCR3A = pwm1a_val;
      OCR3B = 0;
      
      // Front Left
      OCR4A = pwm1a_val;
      OCR4B = 0;
      
      // Front Right
      OCR3C = pwm1a_val;
      OCR1C = 0;
    } else {  // Reverse
      // Back Left
      OCR1A = 0;
      OCR1B = pwm1b_val;
      
      // Back Right
      OCR3A = 0;
      OCR3B = pwm1b_val;
      
      // Front Left
      OCR4A = 0;
      OCR4B = pwm1b_val;
      
      // Front Right
      OCR3C = 0;
      OCR1C = pwm1b_val;
    }
  }
  return(0);
}

// Initialize Ports
void init_PORTS() {
  DDRF = 0x00;
  PORTF = 0x00;
  
  DDRB = ~0b01100000;
  PORTB = 0X00;
  
  DDRA = 0x00;
  PORTA = 0x01;
}

// Initialize ADC
void init_ADC() {
  ADCSRA = (1<<ADEN) | (1<<ADPS2) | (1<<ADPS1) | (1<<ADPS0);
}

// Read 10-bit value from ADC
uint16_t read_ADC(uint8_t channel) {
  ADMUX = 0x00;   // Reset ADMUX Value
  ADMUX = channel;    // Select which channel to read from
  ADMUX |= (1<<REFS0);  // Set reference voltage to 5V
  
  ADCSRA |= (1<<ADSC);  // Start conversion
  while((ADCSRA & (1<<ADIF)) == 0); // Wait for conversion to finish
  ADCSRA |= (1<<ADIF);  // Reset Interrupt Flag
  
  uint16_t result = ADCL; // Read 10-bit value
  result |= (ADCH<<8);
  
  return result;
}

// Initialize PWM1 output 
void init_PWM1(void) { //On digital 11
  DDRB = 0b11100000; //set the OC1A, B, & C pin as output
  TCCR1A = (1<<WGM11) | (1<<COM1A1) | (1<<COM1B1) | (1<<COM1C1);
  TCCR1B = (1<<WGM13) | (1<<WGM11) | (1<<CS11); //Set to pwm mode 14 with prescaler 8
  ICR1 = 5000;  // Set Frequency
  OCR1A = 2500; // Set Default PWM Duty Cycle to 50%
}

// Initialize PWM3 output 
void init_PWM3(void) { //On digital 5
  DDRE = 0b00111000; //set the OC3A, B, & C pin as output
  TCCR3A = (1<<WGM31) | (1<<COM3A1) | (1<<COM3B1) | (1<<COM3C1);
  TCCR3B = (1<<WGM33) | (1<<WGM31) | (1<<CS31); //Set to pwm mode 14 with prescaler 8
  ICR3 = 5000;  // Set Frequency
  OCR3A = 2500; // Set Default PWM Duty Cycle to 50%
}

// Initialize PWM4 output 
void init_PWM4(void) { //On digital 11
  DDRH = 0b00111000; //set the OCR4A, B, & C pin as output
  TCCR4A = (1<<WGM41) | (1<<COM4A1) | (1<<COM4B1);
  TCCR4B = (1<<WGM43) | (1<<CS41);  //Set to pwm mode 14 with prescaler 8
  ICR4 = 5000;  // Set Freq
  OCR4A = 2500; // Set Default PWM Duty Cycle to 50%
}

// External Interrupt
void ext_int0_init() {
  EIMSK |= (1<<INT0);   // Enable interrupt pin on PD3
  EICRA |= (1<<ISC01) | (1<<ISC00);   // Configure for rising edge
}

ISR(INT0_vect) {
  dir = 1;
}
